/*
 * file: pv_types.h
 * This file is part of the libpv library.
 *
 * Copyright 2005 Pengutronix
 *
 * Please send bug reports to bugs@pengutronix.de
 *
 * See COPYING for the status of this software.
 */

#ifndef PV_TYPES_H
#define PV_TYPES_H

#include <libpv_config.h>

#include <sys/types.h>	/* off_t */
#include <pthread.h>

#ifdef HAVE_MQUEUE_H
#include <mqueue.h>
#endif

#include <pv_list.h>

#include <libpv.h>

#define LEVEL_ALL	7
#define LEVEL_DEBUG	6
#define LEVEL_INFO	5
#define LEVEL_NOTICE	4
#define LEVEL_WARN	3
#define LEVEL_ERROR	2
#define LEVEL_CRIT	1

#ifdef DEBUG
#define dprintf(_level, _fmt, _arg...)			\
do {							\
	if (_libpv_debuglevel >= _level) {		\
		fprintf(stdout, _fmt, ##_arg);		\
		fflush(stdout);				\
	}						\
} while (0)
#else
#define dprintf(_level, _fmt, _arg...)			\
do {							\
	if (_level == LEVEL_CRIT ||			\
	    _level == LEVEL_ERROR ||			\
	    _level == LEVEL_WARN)			\
		fprintf(stderr, _fmt, ##_arg);		\
	} while (0)
#endif

extern int _libpv_debuglevel;

#ifndef MIN
#define	MIN(a,b) (((a)<(b))?(a):(b))
#endif

#ifndef MAX
#define	MAX(a,b) (((a)>(b))?(a):(b))
#endif

struct pv_backing_store;
typedef struct pv_backing_store pv_backing_store_t;


struct pv_backing_store {
	int	(*init)		(pv_backing_store_t	*,
				 const char		*filename,
				 size_t			size);

	int	(*exit)		(pv_backing_store_t	*);

	int	(*read)		(pv_backing_store_t	*,
				 off_t			,
				 size_t			);

	int	(*write)	(pv_backing_store_t	*,
				 off_t			,
				 size_t			);

	int	shmid;

	void	*map;		/*
				 * set to NULL for not mmappable backing stores,
				 * the user will be prompted for a shm segment
				 */
	size_t	size;

	void	*priv;
};


union pv_dflt {
	int8_t		s8;
	uint8_t		u8;

	int16_t		s16;
	uint16_t	u16;

	int32_t		s32;
	uint32_t	u32;

	int64_t		s64;
	uint64_t	u64;

	float		f;
	double		d;

	char		*string;

	void		*ptr;
};


union pv_data {
	int8_t		s8[0];
	uint8_t		u8[0];

	int16_t		s16[0];
	uint16_t	u16[0];

	int32_t		s32[0];
	uint32_t	u32[0];

	int64_t		s64[0];
	uint64_t	u64[0];

	float		f[0];
	double		d[0];

	char		string[0];

	void		*ptr[0];
};

typedef enum {
		PVE_ADD_CLIENT,	/* a new client attaches to the daemon */
		PVE_REMOVE_CLIENT,	/* a clients says Good Bye.. */
		PVE_ADD_PV,	/* add notification */
		PVE_REMOVE_PV,	/* remove notification */
		PVE_CHANGED,	/* This variable was changed */
		PVE_SERVER_QUIT	/* Server should end itself */
} msq_types;

struct pve_message {
	msq_types type;
	pid_t pid;
	union {
		off_t offset;	/* for PVE_ADD_PV and PVE_REMOVE_PV, PVE_CHANGED */
	} message;
};

#define QUEUE_NAME "/pve_%d"
#define QUEUE_PERM 666
#define QUEUE_CLIENT "/clnt_%d"

struct pv_engine {
	int			pv_store_size;
	int			pv_sem;		/* SysV semaphore */
	int			pv_sem_key;

	pv_backing_store_t	store;

	/* PV Data */
	struct pv_list		*pv_list;	/* list of process variables */

	/* JVisu */
	struct jv_server	*jv_server;

#ifdef CONFIG_EVENT
	mqd_t			mqdes;		/* Queue to send */
	mqd_t			mq_notify;	/* Queue to receive */
	char			*mq_name;
	int			event_server_key;
#endif
};

/**
 * &pv - declares a process variable in one process image
 * @id: ID of this process variable. Readed from XML tag <port>, attribute "id"
 * @name: Name of this process variable. Readed from XML tag <name>
 *        FIXME: make this a language index array... (rsc)
 * @device: device path (if pv is part of a <portPvChannel> tag)
 *          Readed and concatenated from XML tag <device>, attributes prefix, index and suffix
 * @device_file: file handle to the already open @device
 * @device_read: Update function to interact with external resources
 * @devcie_write: Update function to interact with external resources
 * @type: Type of this process variable. Valid types see enum pv_type
 * @offset: Process variable resides at @offset in the process image
 * @dflt: Default value. Readed from XML tag <pv>, attribute "val"
 * @size: Object size. Readed from XML tag <size>
 * @arraylen: length of the possible array type
 **/
struct pv {
	char 			*id;
	char			*name;
	char			*device;
	int			device_file;

	pv_update_callback_fn_t	device_read;
	pv_update_callback_fn_t	device_write;

	enum pv_type 		type;
	off_t			offset;
	union pv_dflt		dflt;
	int			has_min;
	int			has_max;
	union pv_dflt		min;
	union pv_dflt		max;

	size_t			size;
	size_t			arraylen;
	struct pv		*next;
};

#endif	/* !PV_TYPES_H */
